import 'dart:async';

import 'package:agora_rtc_engine/agora_rtc_engine.dart';
import 'package:agora_rtc_wrapper/agora_rtc_wrapper.dart';
import 'package:flutter/material.dart';

/// Created by taohid on 15,May, 2020
/// Email: taohid32@gmail.com

class AgoraRtcWrapperImpl implements AgoraRtcWrapper {
  bool isInit = false;
  final _users = <int>[];
  bool _muted = false;
  final _oneSec = const Duration(seconds: 1);
  Timer _timer;
  int _totalSecond = 0;

  OnTotalTime _onTotalTime;
  OnError _onError;
  OnJoinSuccess _onJoinSuccess;
  VoidCallback _onLeaveChannel;
  OnUserJoined _onUserJoined;
  OnUserOffline _onUserOffline;
  OnFirstRemoteVideoFrame _onFirstRemoteVideoFrame;

  static final AgoraRtcWrapperImpl _instance = AgoraRtcWrapperImpl._internal();
  AgoraRtcWrapperImpl._internal();
  factory AgoraRtcWrapperImpl() {
    return _instance;
  }

  @override
  void init(String appId) {
    _initAgoraEngine(appId);
  }

  @override
  void listen({
    OnTotalTime onTotalTime,
    OnError onError,
    OnJoinSuccess onJoinSuccess,
    VoidCallback onLeaveChannel,
    OnUserJoined onUserJoined,
    OnUserOffline onUserOffline,
    OnFirstRemoteVideoFrame onFirstRemoteVideoFrame,
  }) {
    if (onTotalTime != null) this._onTotalTime = onTotalTime;
    if (onError != null) this._onError = onError;
    if (onJoinSuccess != null) this._onJoinSuccess = onJoinSuccess;
    if (onLeaveChannel != null) this._onLeaveChannel = onLeaveChannel;
    if (onUserJoined != null) this._onUserJoined = onUserJoined;
    if (onUserOffline != null) this._onUserOffline = onUserOffline;
    if (onFirstRemoteVideoFrame != null)
      this._onFirstRemoteVideoFrame = onFirstRemoteVideoFrame;
  }

  Future _initAgoraEngine(String appId) async {
    if (isInit) return;
    AgoraRtcEngine.create(appId);
    AgoraRtcEngine.enableVideo();
    AgoraRtcEngine.enableAudio();
    AgoraRtcEngine.setParameters(
        '{\"che.video.lowBitRateStreamParameter\":{\"width\":320,\"height\":180,\"frameRate\":15,\"bitRate\":140}}');
    AgoraRtcEngine.setChannelProfile(ChannelProfile.Communication);
    VideoEncoderConfiguration config = VideoEncoderConfiguration();
    config.orientationMode = VideoOutputOrientationMode.FixedPortrait;
    AgoraRtcEngine.setVideoEncoderConfiguration(config);
    _initEventListener();
    isInit = true;
  }

  void _initEventListener() {
    AgoraRtcEngine.onError = (error) {
      _onError != null ? _onError(error) : print('On Error Listener Not Set');
    };

    AgoraRtcEngine.onJoinChannelSuccess = (
      String channel,
      int uid,
      int elapsed,
    ) {
      _onJoinSuccess != null
          ? _onJoinSuccess(channel, uid, elapsed)
          : print('onJoinSuccess Listener Not Set');
    };

    AgoraRtcEngine.onFirstRemoteVideoFrame = (
      int uid,
      int width,
      int height,
      int elapsed,
    ) {
      _onFirstRemoteVideoFrame != null
          ? _onFirstRemoteVideoFrame(uid, width, height, elapsed)
          : print('onFirstRemoteVideoFrame Listener Not Set');
    };

    AgoraRtcEngine.onUserJoined = (int uid, int elapsed) {
      _startTimer();
      _users.add(uid);
      _onUserJoined != null
          ? _onUserJoined(uid, elapsed)
          : print('User Joined Listener Not Set');
    };

    AgoraRtcEngine.onLeaveChannel = () {
      _users.clear();
      _onLeaveChannel != null
          ? _onLeaveChannel()
          : print('Leave Listener Not Set');
    };

    AgoraRtcEngine.onUserOffline = (int uid, int reason) {
      _users.remove(uid);
      _onUserOffline != null
          ? _onUserOffline(uid, reason)
          : print('User Offline Listener Not Set');
    };
  }

  void _startTimer() {
    if (_timer == null) {
      _timer = Timer.periodic(_oneSec, (timer) {
        _totalSecond = _totalSecond + 1;
        if (_onTotalTime != null) _onTotalTime(_totalSecond);
      });
    }
  }

  @override
  List<int> getUsers() {
    return _users;
  }

  @override
  bool isCallActive() {
    return _users.isNotEmpty;
  }

  @override
  void joinChannel(String channelName) async {
    AgoraRtcEngine.joinChannel(null, channelName, null, 0);
  }

  @override
  void enableLog() {
    AgoraRtcEngine.setParameters("{\"rtc.log_filter\": 65535}");
  }

  /// for self view uid is always 0
  @override
  Widget get getSelfVideoView =>
      AgoraRenderWidget(0, local: true, preview: true);

  @override
  Widget getVideoView(int uid) => AgoraRenderWidget(uid);

  @override
  void muteAudio(bool mute) {
    AgoraRtcEngine.muteLocalAudioStream(mute);
    _muted = mute;
  }

  @override
  void switchCamera() {
    AgoraRtcEngine.switchCamera();
  }

  @override
  void endCall({@required OnTotalTime onCallEnd}) {
    _timer.cancel();
    if (onCallEnd != null) {
      onCallEnd(_totalSecond);
    }
    _totalSecond = 0;
    _timer = null;
  }

  @override
  void dispose() {
    AgoraRtcEngine.leaveChannel();
    AgoraRtcEngine.destroy();
    _users.clear();
    isInit = false;
  }

  @override
  Widget getDefaultController({
    @required VoidCallback onCallEnd,
    @required ValueChanged<bool> onMute,
    Color selectedButtonColor = Colors.blueAccent,
  }) {
    return Container(
      alignment: Alignment.bottomCenter,
      padding: const EdgeInsets.symmetric(vertical: 48),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          RawMaterialButton(
            onPressed: () => onMute(_muted),
            child: Icon(
              _muted ? Icons.mic_off : Icons.mic,
              color: _muted ? Colors.white : selectedButtonColor,
              size: 20.0,
            ),
            shape: CircleBorder(),
            elevation: 2.0,
            fillColor: _muted ? selectedButtonColor : Colors.white,
            padding: const EdgeInsets.all(12.0),
          ),
          RawMaterialButton(
            onPressed: onCallEnd,
            child: Icon(
              Icons.call_end,
              color: Colors.white,
              size: 35.0,
            ),
            shape: CircleBorder(),
            elevation: 2.0,
            fillColor: Colors.redAccent,
            padding: const EdgeInsets.all(15.0),
          ),
          RawMaterialButton(
            onPressed: switchCamera,
            child: Icon(
              Icons.switch_camera,
              color: selectedButtonColor,
              size: 20.0,
            ),
            shape: CircleBorder(),
            elevation: 2.0,
            fillColor: Colors.white,
            padding: const EdgeInsets.all(12.0),
          )
        ],
      ),
    );
  }
}
